// DIMEMapper.cpp : Implementation of CDIMEMapper
#include "stdafx.h"
#include "CustomMapper.h"
#include "DIMEMapper.h"

/////////////////////////////////////////////////////////////////////////////
// CDIMEMapper

STDMETHODIMP CDIMEMapper::Init(ISoapTypeMapperFactory * par_Factory, IXMLDOMNode * par_Schema, IXMLDOMNode * par_WSMLNode, enXSDType par_xsdType)
{
	m_spSchema = par_Schema;
    m_spWSML = par_WSMLNode;
    return S_OK;
}
STDMETHODIMP CDIMEMapper::Read(ISoapReader * par_soapreader, IXMLDOMNode * par_Node, BSTR par_encoding, enEncodingStyle par_encodingMode, LONG par_flags, VARIANT * par_var)
{
	if (par_var == NULL)
		return E_POINTER;
   try
    {
        using namespace _com_util;

        CComPtr<IXMLDOMElement> spElem;
        CheckError(m_spWSML.QueryInterface(&spElem));
        CComVariant vValue;
        CComPtr<IPersistStream> spPersist;
        if(FAILED(spElem->getAttribute(L"targetPROGID", &vValue)))
        {
            CheckError(spElem->getAttribute(L"targetClassId", &vValue));
            CheckError(vValue.ChangeType(VT_BSTR));
            CLSID clsid = CLSID_NULL;
            CheckError(CLSIDFromString(vValue.bstrVal, &clsid));
            CheckError(spPersist.CoCreateInstance(clsid));
        }
        else
        {
            CheckError(vValue.ChangeType(VT_BSTR));
            CheckError(spPersist.CoCreateInstance(vValue.bstrVal));
        }
        CComPtr<IReceivedAttachment> spAttach;
        CheckError(par_soapreader->GetReferencedAttachment(par_Node, &spAttach));
        CComVariant vArray;
        CheckError(spAttach->GetAsByteArray(&vArray));
        if(vArray.vt != (VT_ARRAY | VT_UI1) ) _com_issue_error(E_UNEXPECTED);
        VOID* pData = NULL;
        CheckError(SafeArrayAccessData(vArray.parray, &pData));
        CComPtr<IStream> spStm;
        CheckError(CreateStreamOnHGlobal(0, TRUE, &spStm));
        long lBound, uBound;
        CheckError(SafeArrayGetLBound(vArray.parray, 1, &lBound));
        CheckError(SafeArrayGetUBound(vArray.parray, 1, &uBound));
        CheckError(spStm->Write(pData, uBound - lBound + 1, 0));
        LARGE_INTEGER off = { 0 };
        CheckError(spStm->Seek(off, STREAM_SEEK_SET, 0));
        CheckError(spPersist->Load(spStm));
        CComPtr<IUnknown> spUnk;
        CheckError(spPersist.QueryInterface(&spUnk));
        CComVariant vResult = spUnk;
        
        CheckError(vResult.Detach(par_var));
        
   }
   catch(_com_error& e)
   {
        return e.Error();
   }
   return S_OK;
}
STDMETHODIMP CDIMEMapper::Write(ISoapSerializer * par_ISoapSerializer, BSTR par_encoding, enEncodingStyle par_encodingMode, LONG par_flags, VARIANT * par_var)
{
    
    try
    {
        using namespace _com_util;
        if((par_var->vt != VT_UNKNOWN) && (par_var->vt != VT_DISPATCH))
            _com_issue_error(E_INVALIDARG);
        CComPtr<IPersistStream> spPersist;
        CheckError(par_var->punkVal->QueryInterface(IID_IPersistStream, (void**)&spPersist));
        CComPtr<IStream> spStm;
        CheckError(CreateStreamOnHGlobal(0, TRUE, &spStm));
        CheckError(spPersist->Save(spStm, FALSE));
        CComPtr<IStreamAttachment> spAttach;
        CheckError(spAttach.CoCreateInstance(CLSID_StreamAttachment30));
        CheckError(spAttach->putref_Stream(spStm));
        CheckError(par_ISoapSerializer->AddAttachmentAndReference(spAttach));
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}
STDMETHODIMP CDIMEMapper::VarType(LONG * par_Type)
{
	if (par_Type == NULL)
		return E_POINTER;
	*par_Type = VT_UNKNOWN;	
	return S_OK;
}
STDMETHODIMP CDIMEMapper::Iid(BSTR * par_IIDAsString)
{
	if (par_IIDAsString == NULL)
		return E_POINTER;
		
	*par_IIDAsString = CComBSTR(L"").Detach();
    return S_OK;
}
STDMETHODIMP CDIMEMapper::SchemaNode(IXMLDOMNode * * par_SchemaNode)
{
	if (par_SchemaNode == NULL)
		return E_POINTER;
	*par_SchemaNode = CComPtr<IXMLDOMNode>(m_spSchema).Detach();	
	return S_OK;
}
STDMETHODIMP CDIMEMapper::XsdType(enXSDType * par_xsdType)
{
	if (par_xsdType == NULL)
		return E_POINTER;
	*par_xsdType = enXSDbinary;
	return S_OK;
}