// PersistMapper.cpp : Implementation of CPersistMapper
#include "stdafx.h"
#include "CustomMapper.h"
#include "PersistMapper.h"

/////////////////////////////////////////////////////////////////////////////
// CPersistMapper

STDMETHODIMP CPersistMapper::Init(ISoapTypeMapperFactory * par_Factory, IXMLDOMNode * par_Schema, IXMLDOMNode * par_WSMLNode, enXSDType par_xsdType)
{
	m_spSchema = par_Schema;
    m_spWSML = par_WSMLNode;
    return S_OK;
}
STDMETHODIMP CPersistMapper::Read(ISoapReader * par_soapreader, IXMLDOMNode * par_Node, BSTR par_encoding, enEncodingStyle par_encodingMode, LONG par_flags, VARIANT * par_var)
{
	if (par_var == NULL)
		return E_POINTER;
   try
    {
        using namespace _com_util;

        CComPtr<IDataEncoderFactory> spFactory;

        CComPtr<IXMLDOMElement> spElem;
        CheckError(m_spWSML.QueryInterface(&spElem));
        CComVariant vValue;
        CComPtr<IPersistStream> spPersist;
        if(FAILED(spElem->getAttribute(L"targetPROGID", &vValue)))
        {
            CheckError(spElem->getAttribute(L"targetClassId", &vValue));
            CheckError(vValue.ChangeType(VT_BSTR));
            CLSID clsid = CLSID_NULL;
            CheckError(CLSIDFromString(vValue.bstrVal, &clsid));
            CheckError(spPersist.CoCreateInstance(clsid));
        }
        else
        {
            CheckError(vValue.ChangeType(VT_BSTR));
            CheckError(spPersist.CoCreateInstance(vValue.bstrVal));
        }
        CheckError(spFactory.CoCreateInstance(CLSID_DataEncoderFactory30));

        CComPtr<IDataEncoder> spEncoder;
        CheckError(spFactory->GetDataEncoder(CComBSTR(L"base64"), &spEncoder));

        CComBSTR bstrText;
        CheckError(par_Node->get_text(&bstrText));
        
        CComPtr<IStream> spStm;
        USES_CONVERSION;
        CheckError(CreateStreamOnHGlobal(0 , TRUE, &spStm));
        CheckError(spStm->Write(W2A(bstrText), bstrText.Length(), 0));
        LARGE_INTEGER off = {0};
        CheckError(spStm->Seek(off, STREAM_SEEK_SET, 0));
        CComPtr<IStream> spDecoded;
        CheckError(CreateStreamOnHGlobal(0 , TRUE, &spDecoded));
        CheckError(spEncoder->DecodeStream(spStm, spDecoded));
        CheckError(spDecoded->Seek(off, STREAM_SEEK_SET, 0));
        CheckError(spPersist->Load(spDecoded));
        CComPtr<IUnknown> spUnk;
        CheckError(spPersist.QueryInterface(&spUnk));
        CComVariant vResult = spUnk;
        
        CheckError(vResult.Detach(par_var));
   }
   catch(_com_error& e)
   {
        return e.Error();
   }
   return S_OK;
}
STDMETHODIMP CPersistMapper::Write(ISoapSerializer * par_ISoapSerializer, BSTR par_encoding, enEncodingStyle par_encodingMode, LONG par_flags, VARIANT * par_var)
{
    
    try
    {
        using namespace _com_util;

    	CComPtr<IDataEncoderFactory> spFactory;
        CheckError(spFactory.CoCreateInstance(CLSID_DataEncoderFactory30));

        CComPtr<IDataEncoder> spEncoder;
        CheckError(spFactory->GetDataEncoder(CComBSTR(L"base64"), &spEncoder));
        if((par_var->vt != VT_UNKNOWN) && (par_var->vt != VT_DISPATCH))
            _com_issue_error(E_INVALIDARG);
        CComPtr<IPersistStream> spPersist;
        CheckError(par_var->punkVal->QueryInterface(IID_IPersistStream, (void**)&spPersist));
        CComPtr<IStream> spStm;
        CheckError(CreateStreamOnHGlobal(0, TRUE, &spStm));
        CheckError(spPersist->Save(spStm, FALSE));
        LARGE_INTEGER off = {0};
        CheckError(spStm->Seek(off, STREAM_SEEK_SET, NULL));
        STATSTG st = {0};
        CheckError(spStm->Stat(&st, 0));
        CComPtr<IStream> spEncoded;
        CheckError(CreateStreamOnHGlobal(0, TRUE, &spEncoded));
        CheckError(spEncoder->EncodeStream(spStm, spEncoded));
        CheckError(spEncoded->Seek(off, STREAM_SEEK_SET, NULL));
        LPVOID hGlobal = NULL;
        CheckError(GetHGlobalFromStream(spEncoded, &hGlobal));
        CheckError(spEncoded->Stat(&st, 0));
        CheckError(par_ISoapSerializer->WriteBuffer(st.cbSize.LowPart ,*((unsigned char**) hGlobal)));
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}
STDMETHODIMP CPersistMapper::VarType(LONG * par_Type)
{
	if (par_Type == NULL)
		return E_POINTER;
	*par_Type = VT_UNKNOWN;	
	return S_OK;
}
STDMETHODIMP CPersistMapper::Iid(BSTR * par_IIDAsString)
{
	if (par_IIDAsString == NULL)
		return E_POINTER;
		
	*par_IIDAsString = CComBSTR(L"").Detach();
    return S_OK;
}
STDMETHODIMP CPersistMapper::SchemaNode(IXMLDOMNode * * par_SchemaNode)
{
	if (par_SchemaNode == NULL)
		return E_POINTER;
	*par_SchemaNode = CComPtr<IXMLDOMNode>(m_spSchema).Detach();	
	return S_OK;
}
STDMETHODIMP CPersistMapper::XsdType(enXSDType * par_xsdType)
{
	if (par_xsdType == NULL)
		return E_POINTER;
	*par_xsdType = enXSDbase64binary;
	return S_OK;
}