// Connector.cpp : Implementation of CConnector
#include "stdafx.h"
#include "SimpleConnector.h"
#include "Connector.h"

/////////////////////////////////////////////////////////////////////////////
// CConnector
HRESULT CConnector::FinalConstruct()
{
    create_object<CInputStream>(m_pInput);
    m_pInput->init(this);

    create_object<COutputStream>(m_pOutput);
    m_pOutput->init(this);

    create_object<CMemStream>(m_pMemStream);
    


    m_bAsync = false;
    m_nCallInterval = 1000;
    return S_OK;
}


STDMETHODIMP CConnector::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ISoapConnector
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
        if (::InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CConnector::get_InputStream(IStream * * pVal )
{
    ObjectLock lock_(this);
    try
    {
        if(!pVal) _com_issue_error(E_INVALIDARG);
        *pVal = IStreamPtr(m_pInput->GetUnknown()).Detach();
    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::get_OutputStream(IStream * * pVal )
{
    ObjectLock lock_(this);
    try
    {
        if(!pVal) _com_issue_error(E_INVALIDARG);
        *pVal = IStreamPtr(m_pOutput->GetUnknown()).Detach();
    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::get_Property(BSTR pPropertyName,VARIANT * pPropertyValue )
{
    ObjectLock lock_(this);
    try
    {
        if(!wcsicmp(pPropertyName, L"ResponseHTTPHeader"))
        {
            *pPropertyValue = _variant_t(m_spRequest->getAllResponseHeaders()).Detach();
        }
        else
            _com_issue_error(E_INVALIDARG);

    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::put_Property(BSTR pPropertyName,VARIANT pPropertyValue )
{
    ObjectLock lock_(this);
    try
    {
        _variant_t v = pPropertyValue;
        if(!wcsicmp(pPropertyName, L"Async"))
        {
            v.ChangeType(VT_BOOL);
            m_bAsync = (v.boolVal == VARIANT_TRUE);
            // open
            m_spRequest->open(L"POST", (WCHAR*)m_bstrURL, _variant_t(m_bAsync ? VARIANT_TRUE : VARIANT_FALSE ), vtMissing, vtMissing);

        }
        else if(!wcsicmp(pPropertyName, L"CallbackInterval"))
        {
            v.ChangeType(VT_I4);
            m_nCallInterval = v.lVal;
        }
        else if(!wcsicmp(pPropertyName, L"Callback"))
        {
            if((v.vt != VT_UNKNOWN) && (v.vt != VT_DISPATCH))
                _com_issue_error(E_INVALIDARG);
            CComPtr<IAsyncSOAPCall> spClient;

            HRESULT hr = v.punkVal->QueryInterface(IID_IAsyncSOAPCall, (void**)&spClient);
            if(SUCCEEDED(hr))
            {
                m_spClient.Release();
                m_spClient = spClient;
            }
            else
                _com_issue_error(hr);
        }
    
    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::ConnectWSDL(IWSDLPort * pPort )
{
    ObjectLock lock_(this);
    try
    {
        CComBSTR address;

        
        _com_util::CheckError(pPort->get_Address(&address));
        m_bstrURL = address;
    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::Reset( )
{
    HRESULT hr = m_spRequest.CreateInstance(XML::CLSID_XMLHTTP);
    return S_OK;
}

STDMETHODIMP CConnector::BeginMessageWSDL(IWSDLOperation * pOperation )
{
    ObjectLock lock_(this);
    try
    {
        HRESULT hr = m_spRequest.CreateInstance(XML::CLSID_XMLHTTP);
        m_spRequest->open(L"POST", (WCHAR*)m_bstrURL, _variant_t(m_bAsync ? VARIANT_TRUE : VARIANT_FALSE ), vtMissing, vtMissing);
        // set request header
        m_bContentTypeSet = false;
        m_spRequest->setRequestHeader(L"charset", L"UTF-8");
        CComBSTR bstrName;
        _com_util::CheckError(pOperation->get_SoapAction(&bstrName));
        
        m_spRequest->setRequestHeader(L"soapaction", (WCHAR*)bstrName);
    }
    catch(_com_error& e)
    {
        _bstr_t msg = e.Description().length() ? e.Description() : e.ErrorMessage();
        return Error( static_cast<LPCOLESTR>(msg), IID_ISoapConnector);
    }
    return S_OK;
}

STDMETHODIMP CConnector::EndMessage( )
{
    ObjectLock lock_(this);
    m_spStm = 0;
    return S_OK;
}

STDMETHODIMP CConnector::Connect( )
{
    return E_NOTIMPL;
}

STDMETHODIMP CConnector::BeginMessage( )
{
    return E_NOTIMPL;
}

HRESULT CConnector::send(const void* pb, ULONG cb)
{
    ObjectLock lock_(this);
    try
    {
        WCHAR buf[20];
        _ultow(cb, buf, 10);
        if(!m_bContentTypeSet)
            m_spRequest->setRequestHeader(L"content-type", L"text/xml");
        m_spRequest->setRequestHeader(L"content-length", buf);
        
        m_pMemStream->init(const_cast<void*>(pb), cb);
        IUnknownPtr spStm = m_pMemStream;
        m_spRequest->send(_variant_t((IUnknown*)spStm));
        
        while(m_spRequest->readyState != 4)
        {
            VARIANT_BOOL bCancel = VARIANT_FALSE;
            if(m_spClient)
            {
                m_spClient->OnProgress(&bCancel);
                if(bCancel == VARIANT_TRUE)
                {
                    m_spRequest->abort();
                    _com_issue_error(E_ABORT);
                }
            }
            // process MQ
            MSG msg;
            const DWORD dwSleepTime = 100;
            DWORD dwTotal = 0;
            while(dwTotal < m_nCallInterval)
            {
                while(::PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
                {
                    ::TranslateMessage(&msg);
                    ::DispatchMessage(&msg);
                }
                Sleep(dwSleepTime);
                dwTotal += dwSleepTime;
            }
        }

        _variant_t v = m_spRequest->responseStream;
        if((v.vt != VT_UNKNOWN) && (v.vt != VT_DISPATCH))
            _com_issue_error(E_UNEXPECTED);
            
        m_spStm = v.punkVal;
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}

HRESULT CConnector::receive(void* pb, ULONG cb, ULONG* pcbRead)
{
    ObjectLock lock_(this);
    try
    {
        if(m_spStm == NULL)
            return E_FAIL;
        _com_util::CheckError(m_spStm->Read(pb, cb,pcbRead));        
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}

HRESULT CConnector::get_property(const _bstr_t& name, _bstr_t& value)
{
    ObjectLock lock_(this);
    try
    {
        if(!wcsicmp(name ,L"ContentType"))
            value = m_spRequest->getResponseHeader("content-type");
        else
            value = m_spRequest->getResponseHeader(name);
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}

HRESULT CConnector::set_property(const _bstr_t& name, const _bstr_t& value)
{
    ObjectLock lock_(this);
    try
    {
        if(!wcsicmp(name ,L"ContentType"))
        {
            m_bContentTypeSet = true;
            m_spRequest->setRequestHeader("content-type", value);
        }
        else
            m_spRequest->setRequestHeader(name, value);
    }
    catch(_com_error& e)
    {
        return e.Error();
    }
    return S_OK;
}
