// SoapProxy.cpp : Implementation of CSoapProxy
#include "stdafx.h"
#include "VtblProxy.h"
#include "SoapProxy.h"

using namespace _com_util;

/////////////////////////////////////////////////////////////////////////////
// CSoapProxy


STDMETHODIMP CSoapProxy::Initialize(BSTR wsdl, BSTR wsml, BSTR service, BSTR port)
{
    ObjectLock lock_(this);
    try
    {
        CComPtr<IWSDLReader> spWSDLReader;
        CheckError(spWSDLReader.CoCreateInstance(CLSID_WSDLReader30));
        CheckError(spWSDLReader->Load(wsdl, wsml));
        CComPtr<IEnumWSDLService> spES;
        CheckError(spWSDLReader->GetSoapServices(&spES));
        CComPtr<IWSDLService> spSvc;
        CheckError(spES->Find(service, &spSvc));
        CComPtr<IEnumWSDLPorts> spEP;
        CheckError(spSvc->GetSoapPorts(&spEP));
        CComPtr<IWSDLPort> spPort;
        CheckError(spEP->Find(port, &spPort));

        CComPtr<ISoapConnectorFactory> spFactory;
        CheckError(spFactory.CoCreateInstance(CLSID_SoapConnectorFactory30));
        CComPtr<ISoapConnector> spConn;
        CheckError(spFactory->CreatePortConnector(spPort, &spConn));
        
        CComPtr<IMessageComposer> spComp;
        CheckError(spComp.CoCreateInstance(CLSID_DimeComposer30));
        CComPtr<IMessageParser> spPars;
        CheckError(spPars.CoCreateInstance(CLSID_DimeParser30));

        CComPtr<ISoapSerializer> spSer;
        CheckError(spSer.CoCreateInstance(CLSID_SoapSerializer30));
        CComPtr<ISoapReader> spReader;
        CheckError(spReader.CoCreateInstance(CLSID_SoapReader30));

        m_spComposer.Put(spComp);
        m_spParser.Put(spPars);
        m_spSerializer.Put(spSer);
        m_spReader.Put(spReader);
        m_spConnector.Put(spConn);
        m_spPort.Put(spPort);
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }
	return S_OK;
}

STDMETHODIMP CSoapProxy::get_ConnectorProperty(BSTR prop, VARIANT *pVal)
{
    ObjectLock lock_(this);
    try
    {
        CComPtr<ISoapConnector> spConn = m_spConnector.Get();
        CheckError(spConn->get_Property(prop, pVal));

    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }

	return S_OK;
}

STDMETHODIMP CSoapProxy::put_ConnectorProperty(BSTR prop, VARIANT newVal)
{

    ObjectLock lock_(this);
    try
    {
        CComPtr<ISoapConnector> spConn = m_spConnector.Get();
        CheckError(spConn->put_Property(prop, newVal));
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }
	return S_OK;
}

STDMETHODIMP CSoapProxy::get_ProxyProperty(BSTR prop, VARIANT *pVal)
{
    ObjectLock lock_(this);
    try
    {
        _com_issue_error(E_NOTIMPL);
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }

	return S_OK;
}

STDMETHODIMP CSoapProxy::put_ProxyProperty(BSTR prop, VARIANT newVal)
{
    ObjectLock lock_(this);
    try
    {
        if(!wcsicmp(prop, L"ConnectorProgID"))
        {
            CComVariant v(newVal);
            CheckError(v.ChangeType(VT_BSTR));
            CComPtr<ISoapConnector> spConn;
            CheckError(spConn.CoCreateInstance(v.bstrVal));
            m_spConnector.Put(spConn);
        }
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }

	return S_OK;
}

STDMETHODIMP CSoapProxy::GetOperation(BSTR name, IWSDLOperation **ppOp)
{

    ObjectLock lock_(this);
    try
    {
        CComPtr<IWSDLPort> spPort = m_spPort.Get();
        CComPtr<IEnumWSDLOperations> spEO;
        CheckError(spPort->GetSoapOperations(&spEO));
        CComPtr<IWSDLOperation> spOp;
        CheckError(spEO->Find(name, &spOp));
        *ppOp = spOp.Detach();
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }
	return S_OK;
}

STDMETHODIMP CSoapProxy::Execute(IWSDLOperation *pOp)
{

    ObjectLock lock_(this);
    try
    {
        CComPtr<ISoapConnector> spConn = m_spConnector.Get();
        CComBSTR bstrAction;
        CheckError(pOp->get_SoapAction(&bstrAction));
        CheckError(spConn->put_Property(CComBSTR(L"SoapAction"), CComVariant(bstrAction)));
        CComPtr<IWSDLPort> spPort = m_spPort.Get();
        CheckError(spConn->ConnectWSDL(spPort));
        CComPtr<ISoapSerializer> spSer = m_spSerializer.Get();
        CComPtr<IMessageComposer> spComp;
        spComp.CoCreateInstance(CLSID_DimeComposer30);
        //= m_spComposer.Get();
        CComPtr<IStream> spInp, spOutp;
        CComPtr<IUnknown> spUnk;
        CheckError(spConn->get_InputStream(&spInp));
        CheckError(spInp.QueryInterface(&spUnk));
        CheckError(spSer->InitWithComposer( CComVariant(spUnk), spComp));
        CheckError(spSer->StartEnvelope(0, 0, 0));
        CheckError(spSer->StartBody(CComBSTR(L"STANDARD")));
        CheckError(pOp->Save(spSer, VARIANT_TRUE));
        CheckError(spSer->EndBody());
        CheckError(spSer->EndEnvelope());
        CheckError(spSer->Finished());
        CheckError(spConn->EndMessage());

        CComPtr<IMessageParser> spPars = m_spParser.Get();
        CComPtr<ISoapReader> spReader = m_spReader.Get();
        CheckError(spConn->get_OutputStream(&spOutp));
        spUnk.Release();
        CheckError(spOutp.QueryInterface(&spUnk));
        VARIANT_BOOL bSuccess = VARIANT_FALSE;
        CheckError(spReader->LoadWithParser(CComVariant(spUnk), spPars, 0, &bSuccess));
        CheckError(spConn->Reset());
        CheckError(spSer->Reset());
        if(bSuccess != VARIANT_TRUE)
            _com_issue_error(E_FAIL);
        XML::IXMLDOMElementPtr spFault;
        CheckError(spReader->get_FaultDetail(reinterpret_cast<IXMLDOMElement**>(&spFault)));
        if(spFault)
        {
            // error from server
            HRESULT hCode = E_FAIL;
            XML::IXMLDOMNodeListPtr spList = spFault->getElementsByTagName(L"mserror:returnCode");
            if(spList->length)
            {
                hCode = _wtol(spList->item[0]->text);
            }
            _bstr_t description;
            spList = spFault->getElementsByTagName(L"mserror:description");
            if(spList->length)
            {
                description = spList->item[0]->text;
            }
            _bstr_t source;
            spList = spFault->getElementsByTagName(L"mserror:source");
            if(spList->length)
            {
                source = spList->item[0]->text;
            }
            ICreateErrorInfoPtr spCrInfo;
            CheckError(CreateErrorInfo(&spCrInfo));
            spCrInfo->SetDescription(description);
            spCrInfo->SetSource(source);
            IErrorInfoPtr spInfo = spCrInfo;
            _com_error e(hCode, spInfo, true);
            throw e;
        }
        CheckError(pOp->Load( spReader, VARIANT_FALSE ));
    }
    catch(_com_error& e)
    {
        return Error(static_cast<LPCWSTR>(e.Description().length() ? e.Description() : e.ErrorMessage()),
            IID_ISoapProxy, e.Error());
    }
	return S_OK;
}
