	
// SoapProxy.h : Declaration of the CSoapProxy

#ifndef __SOAPPROXY_H_
#define __SOAPPROXY_H_

#include "resource.h"       // main symbols


template<class T>
class CGITHolder
{
public:
    CGITHolder() : m_dwCookie(0)
    {
    }
    void SetGITPtr(CComPtr<IGlobalInterfaceTable> spGIT)
    {
        m_spGIT = spGIT;
    }
    void Put(CComPtr<T> pT)
    {
        Release();
        CComPtr<IUnknown> spUnk;
        _com_util::CheckError(pT.QueryInterface(&spUnk));
        _com_util::CheckError(m_spGIT->RegisterInterfaceInGlobal(spUnk, __uuidof(T), &m_dwCookie));    
    }
    CComPtr<T> Get()
    {
        CComPtr<T> spT;
        _com_util::CheckError(m_spGIT->GetInterfaceFromGlobal(m_dwCookie, __uuidof(T), (void**)&spT));
        return spT;
    }
    void Release()
    {
        if(m_dwCookie)
        {
            m_spGIT->RevokeInterfaceFromGlobal(m_dwCookie);
            m_dwCookie = 0;
        }
    }
    ~CGITHolder()
    {
        Release();
    }
private:
    DWORD m_dwCookie;
    CComPtr<IGlobalInterfaceTable> m_spGIT;
};


/////////////////////////////////////////////////////////////////////////////
// CSoapProxy
class ATL_NO_VTABLE CSoapProxy : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CSoapProxy, &CLSID_SoapProxy>,
	public IDispatchImpl<ISoapProxy, &IID_ISoapProxy, &LIBID_VTBLPROXYLib>,
    public ISupportErrorInfoImpl<&IID_ISoapProxy>
{
public:
	CSoapProxy()
	{
		m_pUnkMarshaler = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SOAPPROXY)
DECLARE_GET_CONTROLLING_UNKNOWN()

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSoapProxy)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(ISoapProxy)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
END_COM_MAP()

	HRESULT FinalConstruct()
	{
        ::CoCreateInstance(CLSID_StdGlobalInterfaceTable, NULL,CLSCTX_INPROC_SERVER, 
            IID_IGlobalInterfaceTable, (void **)&m_spGIT);

        m_spPort.SetGITPtr(m_spGIT);
        m_spConnector.SetGITPtr(m_spGIT);
        m_spSerializer.SetGITPtr(m_spGIT);
        m_spReader.SetGITPtr(m_spGIT);
        m_spComposer.SetGITPtr(m_spGIT);
        m_spParser.SetGITPtr(m_spGIT);

		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}

	void FinalRelease()
	{
		m_pUnkMarshaler.Release();
        m_spGIT.Release();
	}

	CComPtr<IUnknown> m_pUnkMarshaler;

// ISoapProxy
public:
	STDMETHOD(Execute)(/*[in]*/IWSDLOperation* pOp);
	STDMETHOD(GetOperation)(/*[in]*/BSTR name, /*[out,retval]*/IWSDLOperation** ppOp);
	STDMETHOD(get_ProxyProperty)(/*[in]*/BSTR prop, /*[out, retval]*/ VARIANT *pVal);
	STDMETHOD(put_ProxyProperty)(/*[in]*/BSTR prop, /*[in]*/ VARIANT newVal);
	STDMETHOD(get_ConnectorProperty)(/*[in]*/BSTR prop, /*[out, retval]*/ VARIANT *pVal);
	STDMETHOD(put_ConnectorProperty)(/*[in]*/BSTR prop, /*[in]*/ VARIANT newVal);
	STDMETHOD(Initialize)(/*[in]*/BSTR wsdl, /*[in]*/BSTR wsml, /*[in]*/BSTR service, /*[in]*/BSTR port);

    
private:    
    CComPtr<IGlobalInterfaceTable> m_spGIT;
    CGITHolder<IWSDLPort> m_spPort;
    CGITHolder<ISoapConnector> m_spConnector;
    CGITHolder<ISoapSerializer> m_spSerializer;
    CGITHolder<ISoapReader> m_spReader;
    CGITHolder<IMessageComposer> m_spComposer;
    CGITHolder<IMessageParser> m_spParser;
};

#endif //__SOAPPROXY_H_
